
/*
	Craft Pro
	Publisher: Red Hall Development

	Include this file in your projects and make any needed adjustments to your project or just see how it
	works and compile the demo with it. You won't need to understand it all so don't worry.

	This is updated from Craft Basic and is vastly more powerful.
*/


/* All atoms can be crafted if chosen */
atom
	var
		list/components
		list/craftedAt
		list/tools

/* Items can be crafted into things (from the inventory or from anywhere you want) */
obj/Item
	var
		// A list of items you can make using this from the inventory
		list/componentof

	proc
		Craft()
			set category = null
			set src in usr

			// Build a list of what the user might possibly want to craft.
			var/list/possibleAtoms = new/list()
			for(var/atomType in componentof)
				possibleAtoms += new atomType

			// Ask the user which one they want to craft
			var/atom/objectToCraft = input(usr,"What would you like to craft using the [name]?","Crafting") as null | anything in possibleAtoms
			if(objectToCraft == null) return // cancel

			usr.craft(objectToCraft)

	New()
		if(componentof)
			verbs += /obj/Item/proc/Craft


/* You can initiate crafting of any atom by calling this */
mob
	proc
		craft(atom/SOMETHING)

			// Do we have the ability to craft this?
			var/canCraft = SOMETHING.canCraft()

			// Can we craft? Do we pocess the nessersary tools and stations?
			if(!canCraft)
				return 0

			// Do we have all the components?
			var/list/usrComponents = SOMETHING.findComponents()

			if(usrComponents)

				// Go through all the components we found
				for(var/obj/Item/component in usrComponents)

					// Stackable component?
					if(component.stackable)

						// Remove the amount we associated with it earlier
						component.removeAmount(usrComponents[component])

					// It wasn't stackable but maybe we need to del more than one
					else
						for(var/nextUsrComponent = 0, nextUsrComponent < usrComponents[component], nextUsrComponent++)
							var/obj/Item/next = locate(component.type) in usr.contents
							if(next == src)
								next.loc = null
							else
								del next

				SOMETHING.whileCrafting()

				// Now we can give the user the craftable.
				usr.getObject(SOMETHING)

			// We didn't have the components
			else
				var/missingComponents = "\n"
				for(var/componentType in SOMETHING.components)
					var/obj/component = new componentType
					missingComponents += "\t[SOMETHING.components[componentType]]x [component.name]\n"
				missingComponents = copytext(missingComponents, 1, length(missingComponents))

				alert("You were missing something to craft this.\n\nYou need a total of:\n[missingComponents]","Missing Components For [SOMETHING.name]")

				return 0
/*
	Redefine this yourself for any extra rules you want to add
	Don't define new rules here
*/
atom
	proc
		canCraft()

			// Do we not have enough space left to fit something new?
			if(!usr.spaceFor(src))

				/*
					We might have space for the item after crafting is completed. The logic being used is
					that if we have at least one non stackable then it will vanish during crafting leaving
					space for this new item.
				*/
				for(var/componentType in components)
					var/obj/Item/component = new componentType
					if(!component.stackable)
						return 1

				usr.spaceError()
				return 0


			// Do we have all the tools required?
			for(var/toolType in tools)

				var/foundTool = 0
				for(var/obj/Item/object in usr.contents)
					if(object.type == toolType)
						foundTool = 1
						break

				if(!foundTool)
					var/atom/toolNeeded = new toolType
					alert("You will need a [toolNeeded.name] to make that.","Missing [toolNeeded.name]")
					return 0

			// Are we adjacent the required stations?
			for(var/stationType in craftedAt)

				var/foundStation = 0
				for(var/obj/object in oview(1))
					if(stationType == object.type)
						foundStation = 1
						break

				if(!foundStation)
					var/atom/stationNeeded = new stationType
					alert("You will need to be next to a [stationNeeded.name] to make that.","Crafted at [stationNeeded.name]")
					return 0

			return 1

	/*
		You can change this to whatever you want for each
		independant item.
	*/
atom
	proc
		whileCrafting()


/* Returns a list of all components associated with a number which is how many is needed*/
atom
	proc
		findComponents()

			var/list/usrComponents = new/list()

			// Check for all components needed
			for(var/componentType in components)

				// Take a component
				var/obj/Item/componentObj = new componentType

				// How many do we need to find?
				var/amountNeeded = components[componentType]

				// It's stackable
				if(componentObj.stackable)

					/* Check for all stacks of this type in the usr */

					// Go to the smallest 1 first
					var/obj/Item/smallestComponent
					for(var/obj/Item/object in usr)
						if(object.type == componentType && object.amount < object.stackSize)
							smallestComponent = object
							break

					// Is this all we need?
					if(smallestComponent)

						// Was it more than what we needed?
						if(smallestComponent.amount >= amountNeeded)
							usrComponents[smallestComponent] = amountNeeded
							amountNeeded = 0
							continue

						// Was it less than what we needed?
						else
							usrComponents[smallestComponent] = smallestComponent.amount
							amountNeeded -= smallestComponent.amount

					// Nope, go into any other stacks they have
					for(var/obj/Item/object in usr)
						if(object.type == componentType && object != smallestComponent)

							// Was it more than what we needed?
							if(object.amount >= amountNeeded)
								usrComponents[object] = amountNeeded
								amountNeeded = 0
								break

							// Was it less than what we needed?
							else
								usrComponents[object] = object.amount
								amountNeeded -= object.amount

					// After that did we find enough?
					if(amountNeeded > 0)
						return null

				// It's not stackable, so we need to find a number of objects
				else

					// How many of these components are in usr contents?
					var/amountFound = 0
					for(var/object in usr.contents)
						if(istype(object,componentType))
							amountFound ++

					// Not enough
					if(amountFound < amountNeeded)
						return null // Return false (null)

					// Enough
					else
						usrComponents[componentObj] = amountNeeded

				// Continue on, enough of this component type were found

			return usrComponents
