
/*
	Inventory System

	Crafting requires you to have a functioning inventory.

	You will need to use these for your inventory if you don't
	already have a working inventory.

	If you do already have a working inventory then you will
	need to copy over only Crafting.dm and substitute in your
	own variables and proceedures for inventory management.

	If you attempt to compile with only Crafting.dm it will
	give you a series of errors which will tell you what you
	need to change.

	It may very well be easier to simply use this inventory
	system and delete your old one.
*/

// Items being defined as anything that can be picked up
obj/Item
	var
		amount = 1 // How many of these objects are there?
		stackable = 0 // Can you hold one of these objects that represents many?
		stackSize = 50

	New()
		updateSuffix()
		if(stackable)
			verbs += /obj/Item/proc/Pick_Up_Amount
			verbs += /obj/Item/proc/Drop_Amount
		..()

	Del()
		// In an inventory? Make sure to update the inventory containing it!
		if(istype(loc,/mob))
			var/mob/M = loc
			M.inventorySlotsUsed --
		..()

	proc
		setAmount(AMOUNT)
			amount = AMOUNT
			updateSuffix()

		addAmount(AMOUNT)
			amount += AMOUNT
			updateSuffix()

		removeAmount(AMOUNT)
			amount -= AMOUNT
			if(amount <= 0) del src
			updateSuffix()

		updateSuffix()
			if(stackable)
				suffix = "([amount]x /[stackSize])"

	verb
		Pick_Up()
			set category = null
			set src in oview(1)
			usr.getObject(src)

		Drop()
			set category = null
			set src in usr
			usr.loseObject(src,usr.loc)

	proc
		Pick_Up_Amount()
			set category = null
			set src in oview(1)
			var/chosenAmount = input(usr,"How many would you like to pick up? There are [src.amount].","Pick Up Amount","") as num | null
			if(chosenAmount == null) return
			usr.getObject(src,chosenAmount)

		Drop_Amount()
			set category = null
			set src in usr
			var/chosenAmount = input(usr,"How many would you like to drop? There are [src.amount].","Drop Amount","") as num | null
			if(chosenAmount == null) return
			usr.loseObject(src,usr.loc,text2num(chosenAmount))

mob
	var
		inventorySlots = 5
		inventorySlotsUsed = 0

	proc

		/*
			Take the object from where ever it was and put it
			in the usr's inventory.

			Optional amount you're taking if you use stacking.
		*/

		spaceError()
			alert("Not enough space. Clear some space and try again.","Max Inventory Space")

		spaceLeft()
			return inventorySlots - inventorySlotsUsed

		spaceFor(obj/Item/OBJECT)

			// This object is stackable? There might already be some in the inventory?
			if(OBJECT.stackable)

				// Locate a partially full stack of the same kind already in the inventory
				var/obj/Item/sameObject
				for(var/obj/Item/object in contents)
					if(object.type == OBJECT.type && object.amount != object.stackSize)
						sameObject = object
						break

				if(sameObject)

					var/freeSpace = sameObject.stackSize - sameObject.amount

					// There is not enough space to fit all incoming into this stack?
					if(OBJECT.amount > freeSpace)
						if(spaceLeft() > 0) return 1
						else return 0

					// There is enough space to fit all incoming into this stack?
					else
						return 1
				else
					return spaceLeft()

			// It's not stackable
			else
				return spaceLeft()

		getObject(obj/Item/OBJECT,AMOUNT)

			// Don't let a person get more items than are availible
			if(AMOUNT > OBJECT.amount)
				AMOUNT = OBJECT.amount

			else if(AMOUNT < 1 && AMOUNT != null)
				return 0

			// This object is stackable? There might already be some in the inventory?
			if(OBJECT.stackable)

				// Locate a partially full stack of the same kind already in the inventory
				var/obj/Item/sameObject
				for(var/obj/Item/object in contents)
					if(object.type == OBJECT.type && object.amount != object.stackSize)
						sameObject = object
						break

				if(sameObject)

					var/freeSpace = sameObject.stackSize - sameObject.amount

					// We're adding a specified amount
					if(AMOUNT)
						if(AMOUNT > freeSpace)
							sameObject.addAmount(freeSpace)
							OBJECT.removeAmount(freeSpace)
							AMOUNT -= freeSpace
						else
							sameObject.addAmount(AMOUNT)
							OBJECT.removeAmount(AMOUNT)
							return 1

					// We're adding all that we can because no amount was specified
					else

						// There is not enough space to fit all incoming into this stack?
						if(OBJECT.amount > freeSpace)
							sameObject.addAmount(freeSpace)
							OBJECT.removeAmount(freeSpace)

						// There is enough space to fit all incoming into this stack?
						else
							sameObject.addAmount(OBJECT.amount)
							del(OBJECT)
							return 1

			// The object can just be added normally
			if(inventorySlotsUsed < inventorySlots)

				// Move specified amount to the inventory
				if(AMOUNT)
					var/obj/Item/clone = new OBJECT.type
					clone.setAmount(AMOUNT)
					OBJECT.removeAmount(AMOUNT)
					contents += clone

				// Move the whole object to the inventory
				else
					contents += OBJECT

				inventorySlotsUsed ++

			else
				spaceError()
				return 0

	proc

		/*
			Take the object from the usr's inventory and move it
			to the location specified.

			Optional amount you're removing if you use stacking.
		*/

		loseObject(obj/Item/OBJECT,LOCATION,AMOUNT)

			// Don't let a person drop more items than are availible
			if(AMOUNT > OBJECT.amount)
				AMOUNT = OBJECT.amount

			else if(AMOUNT < 1 && AMOUNT != null)
				return 0

			// This object is stackable? There might already be some on the ground?
			if(OBJECT.stackable)

				// Try to add it's amount to an object of the same type already in there
				var/obj/Item/sameObject
				for(var/obj/Item/object in LOCATION)
					if(object.type == OBJECT.type && object.amount != object.stackSize)
						sameObject = object
						break

				if(sameObject)

					var/freeSpace = sameObject.stackSize - sameObject.amount

					// We're adding a specified amount
					if(AMOUNT)
						if(AMOUNT > freeSpace)
							sameObject.addAmount(freeSpace)
							OBJECT.removeAmount(freeSpace)
							AMOUNT -= freeSpace
						else
							sameObject.addAmount(AMOUNT)
							OBJECT.removeAmount(AMOUNT)
							return 1

					// We're adding all that we can because no amount was specified
					else

						// There is not enough space to fit all incoming into this stack?
						if(OBJECT.amount > freeSpace)
							sameObject.addAmount(freeSpace)
							OBJECT.removeAmount(freeSpace)

						// There is enough space to fit all incoming into this stack?
						else
							sameObject.addAmount(OBJECT.amount)
							del(OBJECT)
							return 1

			// Move specified amount to location
			if(AMOUNT)
				var/obj/Item/clone = new OBJECT.type
				clone.setAmount(AMOUNT)
				OBJECT.removeAmount(AMOUNT)
				clone.loc = LOCATION

			// Move the full amount because not amount was specified
			else
				OBJECT.loc = LOCATION
				inventorySlotsUsed --

			return 1
